# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that defines the behavior of the
add dialog.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class AddDialog(object):
	"""
	This class defines the behavior of the add dialog.
	"""

	def __init__(self, manager, player):
		"""
		Initialize object.

		@param self: Reference to the AddDialog instance.
		@type self: A AddDialog object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player.
		@type player: An Player object.
		"""
		self.__init_attributes(manager, player)
		self.__set_properties()
		self.__signal_id_1 = manager.connect("destroy", self.__destroy_cb)
		self.__signal_id_2 = self.__dialog.connect("delete-event", self.__delete_event_cb)
		self.__signal_id_3 = self.__dialog.connect("response", self.__response_cb)
		self.__signal_id_4 = self.__dialog.connect("selection-changed", self.__selection_changed_cb)
		self.__signal_id_5 = self.__dialog.connect("file-activated", self.__file_activated_cb)
		self.__signal_id_6 = self.__button.connect("clicked", self.__clicked_cb)

	def __init_attributes(self, manager, player):
		"""
		Initialize data attributes.

		@param self: Reference to the AddDialog instance.
		@type self: A AddDialog object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player.
		@type player: An Player object.
		"""
		from os.path import join, split
		current_folder = split(globals()["__file__"])[0]
		glade_file = join(current_folder, "Stations.glade")
		from gtk.glade import XML
		glade = XML(glade_file, "AddDialog", domain="striim")
		self.__dialog = glade.get_widget("AddDialog")
		self.__button = glade.get_widget("DialogAddButton")
		self.__manager = manager
		self.__player = player
		self.__signal_id_1 = self.__signal_id_2 = self.__signal_id_3 = None
		self.__signal_id_4 = self.__signal_id_5 = self.__signal_id_6 = None
		return

	def __set_properties(self):
		"""
		Set default properties.

		@param self: Reference to the AddDialog instance.
		@type self: An AddDialog object.
		"""
		self.__dialog.set_select_multiple(True)
		self.__dialog.set_property("icon-name", "stock_open")
		self.__dialog.set_transient_for(self.__manager.glade.get_widget("StationWindow"))
		self.__dialog.add_filter(self.__create_shoutcast_filter())
		from Striim.Global import home_folder, desktop_folder
		from os import path
		if path.exists(desktop_folder):
			self.__dialog.set_current_folder(desktop_folder)
		else:
			self.__dialog.set_current_folder(home_folder)
		return

	def __create_shoutcast_filter(self):
		"""
		Create filter for the dialog.

		@param self: Reference to the AddDialog instance.
		@type self: An AddDialog object.

		@return: A file filter.
		@rtype: A gtk.FileFilter object.
		"""
		from gtk import FileFilter
		filefilter = FileFilter()
		filefilter.set_name("Shoutcast File")
		filefilter.add_pattern("*.pls")
		return filefilter

	def __parser_selection(self):
		uris = self.__dialog.get_uris()
		self.__player.parse_playlist(uris)
		self.__hide()
		return

	def show_dialog(self):
		"""
		Show the add dialog.

		@param self: Reference to the AddDialog instance.
		@type self: An AddDialog object.
		"""
		self.__player.response()
		self.__dialog.run()
		self.__player.response()
		return

	def __hide(self):
		"""
		Hide the dialog.

		@param self: Reference to the AddDialog instance.
		@type self: An AddDialog object.
		"""
		self.__player.response()
		self.__dialog.hide()
		self.__player.response()
		return

	def __destroy_cb(self, manager):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the AddDialog instance.
		@type self: An AddDialog object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__signal_id_1, manager)
		disconnect_signal(self.__signal_id_2, self.__dialog)
		disconnect_signal(self.__signal_id_3, self.__dialog)
		disconnect_signal(self.__signal_id_4, self.__dialog)
		disconnect_signal(self.__signal_id_5, self.__dialog)
		disconnect_signal(self.__signal_id_6, self.__button)
		self.__button.destroy()
		self.__dialog.destroy()
		delete_attributes(self)
		self = None
		del self
		return

	def __delete_event_cb(self, *args):
		"""
		Handles callback when the "delete-event" signal is emitted.

		@param self: Reference to the AddDialog instance.
		@type self: An AddDialog object.
		"""
		self.__hide()
		return True

	def __response_cb(self, *args):
		"""
		Handles callback when the "response" signal is emitted.

		@param self: Reference to the AddDialog instance.
		@type self: An AddDialog object.

		@param dialog: Reference to the AddDialog instance.
		@type dialog: An AddDialog object.

		@param response_id: A response identification.
		@type response_id: A Integer object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.__hide()
		return True

	def __selection_changed_cb(self, filechooser):
		"""
		Handles callback when the "selection-changed" signal is emitted.

		@param self: Reference to the AddDialog instance.
		@type self: A AddDialog object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		filename = filechooser.get_filename()
		if filename is None: return
		from os import path
		if path.isdir(filename):
			self.__button.set_property("sensitive", False)
		else:
			self.__button.set_property("sensitive", True)
		return True

	def __file_activated_cb(self, *args):
		"""
		Handles callback when the "file-activated" signal is emitted.

		@param self: Reference to the AddDialog instance.
		@type self: A AddDialog object.

		@return: True to propagate signals to parent widgets.
		@type: A Boolean Object.
		"""
		self.__parser_selection()
		return True

	def __clicked_cb(self, *args):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the AddDialog instance.
		@type self: A AddDialog object.
		"""
		self.__parser_selection()
		return True
